@@ -20,6 +20,7 @@
  ***************************************************************************/
 
 #include "tasks.h"
+#include "support/draghelper.h"
 #include "support/textlabel.h"
 #include "support/tooltip.h"
 
@@ -95,6 +96,7 @@ void Tasks::init()
     qmlRegisterType<TextLabel>( "Tasks", 0, 1, "TextLabel" );
     qmlRegisterType<ToolTipProxy>( "Tasks", 0, 1, "ToolTip" );
     rootContext->setContextProperty("tasksModel", QVariant::fromValue(static_cast<QObject *>(m_tasksModel)));
+    rootContext->setContextProperty("dragHelper", QVariant::fromValue(static_cast<QObject *>(new DragHelper(this))));
 
     // NOTE: This can go away once Plasma::Location becomes available (i.e. once this is
     // a pure-QML applet.)
@@ -299,13 +301,19 @@ void Tasks::itemMove(int id, int newIndex)
 
 void Tasks::itemGeometryChanged(int id, int x, int y, int width, int height)
 {
-    TaskManager::TaskItem* taskItem = static_cast<TaskManager::TaskItem*>(m_groupManager->rootGroup()->getMemberById(id));
+    TaskManager:: AbstractGroupableItem *item = m_groupManager->rootGroup()->getMemberById(id);
 
-    if (!taskItem || !taskItem->task() || !scene())
+    if (!item || item->itemType() != TaskManager::TaskItemType || !scene())
     {
         return;
     }
 
+    TaskManager::TaskItem *taskItem = static_cast<TaskManager::TaskItem *>(item);
+
+    if (!taskItem->task()) {
+        return;
+    }
+
     QGraphicsView *parentView = 0;
     QGraphicsView *possibleParentView = 0;
     // The following was taken from Plasma::Applet, it doesn't make sense to make the item an applet, and this was the easiest way around it.
